""" Functions of evaluation for images and videos."""


def miu(gt, pred):
    """ calculate Mean IoU for groundtruth image and predicted image.
    Args:
        gt (np.ndarray) : groud truth image, shape [h, w]
        pred (np.ndarray) : predicted result image, shape [h, w]
    Returns:
        A float value between [0, 1]
    """
    pass
